@@ -5,7 +5,6 @@ try:
     from hashlib import sha1
 except ImportError:
     from sha import sha as sha1
-from sorl.thumbnail.fields import ThumbnailField
 
 from django.db import models
 from django.db.models import F, Sum
@@ -16,9 +15,9 @@ from django.utils.translation import ugettext_lazy as _
 from django.conf import settings
 
 from pybb.markups import mypostmarkup
-from pybb.fields import AutoOneToOneField
+from pybb.fields import AutoOneToOneField, JSONField
 from pybb.util import urlize, memoize_method, unescape
-from pybb import settings as pybb_settings
+
 
 TZ_CHOICES = [(float(x[0]), x[1]) for x in (
     (-12, '-12'), (-11, '-11'), (-10, '-10'), (-9.5, '-09.5'), (-9, '-09'),
@@ -71,6 +70,7 @@ class Forum(models.Model):
     moderators = models.ManyToManyField(User, blank=True, null=True, verbose_name=_('Moderators'))
     updated = models.DateTimeField(_('Updated'), null=True)
     post_count = models.IntegerField(_('Post count'), blank=True, default=0)
+    topic_count = models.IntegerField(_('Topic count'), blank=True, default=0)
 
     class Meta:
         ordering = ['position']
@@ -80,9 +80,6 @@ class Forum(models.Model):
     def __unicode__(self):
         return self.name
 
-    def topic_count(self):
-        return self.topics.all().count()
-
     def update_post_count(self):
         self.post_count = Topic.objects.filter(forum=self).aggregate(
                                         Sum("post_count"))['post_count__sum']
@@ -140,24 +137,10 @@ class Topic(models.Model):
             self.created = datetime.now()
         super(Topic, self).save(*args, **kwargs)
 
-    def update_read(self, user):
-        read, new = Read.objects.get_or_create(user=user, topic=self)
-        if not new:
-            read.time = datetime.now()
-            read.save()
-
     def update_post_count(self):
         self.post_count = self.posts.count()
         self.save()
 
-    #def has_unreads(self, user):
-        #try:
-            #read = Read.objects.get(user=user, topic=self)
-        #except Read.DoesNotExist:
-            #return True
-        #else:
-            #return self.updated > read.time
-
 
 class RenderableItem(models.Model):
     """
@@ -190,7 +173,7 @@ class Post(RenderableItem):
     user = models.ForeignKey(User, related_name='posts', verbose_name=_('User'))
     created = models.DateTimeField(_('Created'), blank=True)
     updated = models.DateTimeField(_('Updated'), blank=True, null=True)
-    markup = models.CharField(_('Markup'), max_length=15, default=pybb_settings.DEFAULT_MARKUP, choices=MARKUP_CHOICES)
+    markup = models.CharField(_('Markup'), max_length=15, default=settings.PYBB_DEFAULT_MARKUP, choices=MARKUP_CHOICES)
     body = models.TextField(_('Message'))
     body_html = models.TextField(_('HTML version'))
     body_text = models.TextField(_('Text version'))
@@ -215,7 +198,7 @@ class Post(RenderableItem):
             self.created = now
         self.render()
 
-        new = self.id is None
+        new = self.pk is None
 
         super(Post, self).save(*args, **kwargs)
 
@@ -250,23 +233,16 @@ BAN_STATUS = (
 
 class Profile(models.Model):
     user = AutoOneToOneField(User, related_name='pybb_profile', verbose_name=_('User'))
-    site = models.URLField(_('Site'), verify_exists=False, blank=True)
-    jabber = models.CharField(_('Jabber'), max_length=80, blank=True)
-    icq = models.CharField(_('ICQ'), max_length=12, blank=True)
-    msn = models.CharField(_('MSN'), max_length=80, blank=True)
-    aim = models.CharField(_('AIM'), max_length=80, blank=True)
-    yahoo = models.CharField(_('Yahoo'), max_length=80, blank=True)
-    location = models.CharField(_('Location'), max_length=30, blank=True)
-    signature = models.TextField(_('Signature'), blank=True, max_length=pybb_settings.SIGNATURE_MAX_LENGTH)
-    signature_html = models.TextField(_('Signature HTML Version'), blank=True, max_length=pybb_settings.SIGNATURE_MAX_LENGTH+30)
-    time_zone = models.FloatField(_('Time zone'), choices=TZ_CHOICES, default=float(pybb_settings.DEFAULT_TIME_ZONE))
+    signature = models.TextField(_('Signature'), blank=True, max_length=settings.PYBB_SIGNATURE_MAX_LENGTH)
+    signature_html = models.TextField(_('Signature HTML Version'), blank=True, max_length=settings.PYBB_SIGNATURE_MAX_LENGTH+30)
+    time_zone = models.FloatField(_('Time zone'), choices=TZ_CHOICES, default=float(settings.PYBB_DEFAULT_TIME_ZONE))
     language = models.CharField(_('Language'), max_length=10, blank=True,
                                 choices=settings.LANGUAGES)
-    avatar = ThumbnailField(_('Avatar'), blank=True, upload_to=pybb_settings.AVATARS_UPLOAD_TO, size=(pybb_settings.AVATAR_WIDTH, pybb_settings.AVATAR_HEIGHT))
     show_signatures = models.BooleanField(_('Show signatures'), blank=True, default=True)
-    markup = models.CharField(_('Default markup'), max_length=15, default=pybb_settings.DEFAULT_MARKUP, choices=MARKUP_CHOICES)
+    markup = models.CharField(_('Default markup'), max_length=15, default=settings.PYBB_DEFAULT_MARKUP, choices=MARKUP_CHOICES)
     ban_status = models.SmallIntegerField(_('Ban status'), default=0, choices=BAN_STATUS)
     ban_till = models.DateTimeField(_('Ban till'), blank=True, null=True, default=None)
+    post_count = models.IntegerField(_('Post count'), blank=True, default=0)
 
     class Meta:
         verbose_name = _('Profile')
@@ -287,37 +263,8 @@ class Profile(models.Model):
             return True
         return False
 
-    @memoize_method
-    def post_count(self):
-        """ Use from template, in future may be replace by normalize field
-
-        {% trans 'Posts' %}: {{ post.user.pybb_profile.post_count }}"""
-        return self.user.posts.all().count()
-
-
-class Read(models.Model):
-    """
-    For each topic that user has entered the time
-    is logged to this model.
-    """
-
-    user = models.ForeignKey(User, verbose_name=_('User'))
-    topic = models.ForeignKey(Topic, verbose_name=_('Topic'))
-    time = models.DateTimeField(_('Time'), blank=True)
-
-    class Meta:
-        unique_together = ['user', 'topic']
-        verbose_name = _('Read')
-        verbose_name_plural = _('Reads')
-
-    def save(self, *args, **kwargs):
-        if self.time is None:
-            self.time = datetime.now()
-        super(Read, self).save(*args, **kwargs)
-
-
-    def __unicode__(self):
-        return u'T[%d], U[%d]: %s' % (self.topic.id, self.user.id, unicode(self.time))
+    def get_absolute_url(self):
+        return reverse('pybb_profile', args=[self.user.username])
 
 
 class Attachment(models.Model):
@@ -351,7 +298,7 @@ class Attachment(models.Model):
 
 
     def get_absolute_path(self):
-        return os.path.join(settings.MEDIA_ROOT, pybb_settings.ATTACHMENT_UPLOAD_TO,
+        return os.path.join(settings.MEDIA_ROOT, settings.PYBB_ATTACHMENT_UPLOAD_TO,
                             self.path)
 
     class Meta:
@@ -359,6 +306,30 @@ class Attachment(models.Model):
         verbose_name_plural = _('Attachments')
 
 
+class ReadTracking(models.Model):
+    """
+    Model for tracking read/unread posts.
+
+    `topics` field stores JSON serialized mapping of
+    `topic pk` --> `topic last post pk`
+    """
+
+    user = AutoOneToOneField(User)
+    topics = JSONField(null=True)
+    last_read = models.DateTimeField(null=True)
+
+    class Meta:
+        verbose_name = _('Read tracking')
+        verbose_name_plural = _('Read tracking')
+
+    def __unicode__(self):
+        return self.user.username
+
+    def save(self, *args, **kwargs):
+        if not self.pk:
+            self.last_read = datetime.now()
+        super(ReadTracking, self).save(*args, **kwargs)
+
 
 from pybb import signals
 signals.setup_signals()
